<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}
require_once _PS_MODULE_DIR_ . 'payphone/lib/api/PayphoneButton.php';
require_once _PS_MODULE_DIR_ . 'payphone/lib/Configuration.php';
require_once _PS_MODULE_DIR_ . 'payphone/PaymentType.php';

/**
 * Clase que define el módulo de pago PayPhone
 */
use PrestaShop\PrestaShop\Adapter\StockManager;
class PayPhone extends PaymentModule {

    protected $_html = '';
    protected $_postErrors = array();
    public $token;
    public $store;
    public $test;
    private $config = array('PAYPHONE_TOKEN', 'PAYPHONE_STORE');
    private $config_vars = array(
        'PAYPHONE_TOKEN' => 'Token is required.',
    );
    private $payphone_pending = array(
        'PS_PAYPHONE_PENDING' => array(
            'es' => 'Pendiente de pago con Payphone',
            'en' => 'Pending Payphone payment'));
    private $payphone_green = array(
        'PS_PAYPHONE_APPROVED' => array(
            'es' => 'Pago con Payphone aprobado',
            'en' => 'Accepted Payphone payment'),
    );
    private $payphone_red = array(
        'PS_PAYPHONE_REJECTED' => array(
            'es' => 'Pago con Payphone cancelado',
            'en' => 'Canceled or error Payphone payment'),
    );

    public function validateOrder(
    $id_cart, $id_order_state, $amount_paid, $payment_method = 'Unknown', $message = null, $extra_vars = array(), $currency_special = null, $dont_touch_amount = false, $secure_key = false, Shop $shop = null
    ) {
        if (self::DEBUG_MODE) {
            PrestaShopLogger::addLog('PaymentModule::validateOrder - Function called', 1, null, 'Cart', (int) $id_cart, true);
        }

        if (!isset($this->context)) {
            $this->context = Context::getContext();
        }
        $this->context->cart = new Cart((int) $id_cart);
        $this->context->customer = new Customer((int) $this->context->cart->id_customer);
        // The tax cart is loaded before the customer so re-cache the tax calculation method
        $this->context->cart->setTaxCalculationMethod();

        $this->context->language = new Language((int) $this->context->cart->id_lang);
        $this->context->shop = ($shop ? $shop : new Shop((int) $this->context->cart->id_shop));
        ShopUrl::resetMainDomainCache();
        $id_currency = $currency_special ? (int) $currency_special : (int) $this->context->cart->id_currency;
        $this->context->currency = new Currency((int) $id_currency, null, (int) $this->context->shop->id);
        if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_delivery') {
            $context_country = $this->context->country;
        }

        $order_status = new OrderState((int) $id_order_state, (int) $this->context->language->id);
        if (!Validate::isLoadedObject($order_status)) {
            PrestaShopLogger::addLog('PaymentModule::validateOrder - Order Status cannot be loaded', 3, null, 'Cart', (int) $id_cart, true);
            throw new PrestaShopException('Can\'t load Order status');
        }

        if (!$this->active) {
            PrestaShopLogger::addLog('PaymentModule::validateOrder - Module is not active', 3, null, 'Cart', (int) $id_cart, true);
            die(Tools::displayError());
        }

        // Does order already exists ?
        if (Validate::isLoadedObject($this->context->cart) && $this->context->cart->OrderExists() == false) {
            if ($secure_key !== false && $secure_key != $this->context->cart->secure_key) {
                PrestaShopLogger::addLog('PaymentModule::validateOrder - Secure key does not match', 3, null, 'Cart', (int) $id_cart, true);
                die(Tools::displayError());
            }

            // For each package, generate an order
            $delivery_option_list = $this->context->cart->getDeliveryOptionList();
            $package_list = $this->context->cart->getPackageList();
            $cart_delivery_option = $this->context->cart->getDeliveryOption();

            // If some delivery options are not defined, or not valid, use the first valid option
            foreach ($delivery_option_list as $id_address => $package) {
                if (!isset($cart_delivery_option[$id_address]) || !array_key_exists($cart_delivery_option[$id_address], $package)) {
                    foreach ($package as $key => $val) {
                        $cart_delivery_option[$id_address] = $key;
                        break;
                    }
                }
            }

            $order_list = array();
            $order_detail_list = array();

            do {
                $reference = Order::generateReference();
            } while (Order::getByReference($reference)->count());

            $this->currentOrderReference = $reference;

            $cart_total_paid = (float) Tools::ps_round((float) $this->context->cart->getOrderTotal(true, Cart::BOTH), 2);

            foreach ($cart_delivery_option as $id_address => $key_carriers) {
                foreach ($delivery_option_list[$id_address][$key_carriers]['carrier_list'] as $id_carrier => $data) {
                    foreach ($data['package_list'] as $id_package) {
                        // Rewrite the id_warehouse
                        $package_list[$id_address][$id_package]['id_warehouse'] = (int) $this->context->cart->getPackageIdWarehouse($package_list[$id_address][$id_package], (int) $id_carrier);
                        $package_list[$id_address][$id_package]['id_carrier'] = $id_carrier;
                    }
                }
            }
            // Make sure CartRule caches are empty
            CartRule::cleanCache();
            $cart_rules = $this->context->cart->getCartRules();
            foreach ($cart_rules as $cart_rule) {
                if (($rule = new CartRule((int) $cart_rule['obj']->id)) && Validate::isLoadedObject($rule)) {
                    if ($error = $rule->checkValidity($this->context, true, true)) {
                        $this->context->cart->removeCartRule((int) $rule->id);
                        if (isset($this->context->cookie) && isset($this->context->cookie->id_customer) && $this->context->cookie->id_customer && !empty($rule->code)) {
                            Tools::redirect('index.php?controller=order&submitAddDiscount=1&discount_name=' . urlencode($rule->code));
                        } else {
                            $rule_name = isset($rule->name[(int) $this->context->cart->id_lang]) ? $rule->name[(int) $this->context->cart->id_lang] : $rule->code;
                            $error = $this->trans('The cart rule named "%1s" (ID %2s) used in this cart is not valid and has been withdrawn from cart', array($rule_name, (int) $rule->id), 'Admin.Payment.Notification');
                            PrestaShopLogger::addLog($error, 3, '0000002', 'Cart', (int) $this->context->cart->id);
                        }
                    }
                }
            }

            foreach ($package_list as $id_address => $packageByAddress) {
                foreach ($packageByAddress as $id_package => $package) {
                    /** @var Order $order */
                    $order = new Order();
                    $order->product_list = $package['product_list'];

                    if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_delivery') {
                        $address = new Address((int) $id_address);
                        $this->context->country = new Country((int) $address->id_country, (int) $this->context->cart->id_lang);
                        if (!$this->context->country->active) {
                            throw new PrestaShopException('The delivery address country is not active.');
                        }
                    }

                    $carrier = null;
                    if (!$this->context->cart->isVirtualCart() && isset($package['id_carrier'])) {
                        $carrier = new Carrier((int) $package['id_carrier'], (int) $this->context->cart->id_lang);
                        $order->id_carrier = (int) $carrier->id;
                        $id_carrier = (int) $carrier->id;
                    } else {
                        $order->id_carrier = 0;
                        $id_carrier = 0;
                    }

                    $order->id_customer = (int) $this->context->cart->id_customer;
                    $order->id_address_invoice = (int) $this->context->cart->id_address_invoice;
                    $order->id_address_delivery = (int) $id_address;
                    $order->id_currency = $this->context->currency->id;
                    $order->id_lang = (int) $this->context->cart->id_lang;
                    $order->id_cart = (int) $this->context->cart->id;
                    $order->reference = $reference;
                    $order->id_shop = (int) $this->context->shop->id;
                    $order->id_shop_group = (int) $this->context->shop->id_shop_group;

                    $order->secure_key = ($secure_key ? pSQL($secure_key) : pSQL($this->context->customer->secure_key));
                    $order->payment = $payment_method;
                    if (isset($this->name)) {
                        $order->module = $this->name;
                    }
                    $order->recyclable = $this->context->cart->recyclable;
                    $order->gift = (int) $this->context->cart->gift;
                    $order->gift_message = $this->context->cart->gift_message;
                    $order->mobile_theme = $this->context->cart->mobile_theme;
                    $order->conversion_rate = $this->context->currency->conversion_rate;
                    $amount_paid = !$dont_touch_amount ? Tools::ps_round((float) $amount_paid, 2) : $amount_paid;
                    $order->total_paid_real = 0;

                    $order->total_products = (float) $this->context->cart->getOrderTotal(false, Cart::ONLY_PRODUCTS, $order->product_list, $id_carrier);
                    $order->total_products_wt = (float) $this->context->cart->getOrderTotal(true, Cart::ONLY_PRODUCTS, $order->product_list, $id_carrier);
                    $order->total_discounts_tax_excl = (float) abs($this->context->cart->getOrderTotal(false, Cart::ONLY_DISCOUNTS, $order->product_list, $id_carrier));
                    $order->total_discounts_tax_incl = (float) abs($this->context->cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS, $order->product_list, $id_carrier));
                    $order->total_discounts = $order->total_discounts_tax_incl;

                    $order->total_shipping_tax_excl = (float) $this->context->cart->getPackageShippingCost((int) $id_carrier, false, null, $order->product_list);
                    $order->total_shipping_tax_incl = (float) $this->context->cart->getPackageShippingCost((int) $id_carrier, true, null, $order->product_list);
                    $order->total_shipping = $order->total_shipping_tax_incl;

                    if (!is_null($carrier) && Validate::isLoadedObject($carrier)) {
                        $order->carrier_tax_rate = $carrier->getTaxesRate(new Address((int) $this->context->cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')}));
                    }

                    $order->total_wrapping_tax_excl = (float) abs($this->context->cart->getOrderTotal(false, Cart::ONLY_WRAPPING, $order->product_list, $id_carrier));
                    $order->total_wrapping_tax_incl = (float) abs($this->context->cart->getOrderTotal(true, Cart::ONLY_WRAPPING, $order->product_list, $id_carrier));
                    $order->total_wrapping = $order->total_wrapping_tax_incl;

                    $order->total_paid_tax_excl = (float) Tools::ps_round((float) $this->context->cart->getOrderTotal(false, Cart::BOTH, $order->product_list, $id_carrier), _PS_PRICE_COMPUTE_PRECISION_);
                    $order->total_paid_tax_incl = (float) Tools::ps_round((float) $this->context->cart->getOrderTotal(true, Cart::BOTH, $order->product_list, $id_carrier), _PS_PRICE_COMPUTE_PRECISION_);
                    $order->total_paid = $order->total_paid_tax_incl;
                    $order->round_mode = Configuration::get('PS_PRICE_ROUND_MODE');
                    $order->round_type = Configuration::get('PS_ROUND_TYPE');

                    $order->invoice_date = '0000-00-00 00:00:00';
                    $order->delivery_date = '0000-00-00 00:00:00';

                    if (self::DEBUG_MODE) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - Order is about to be added', 1, null, 'Cart', (int) $id_cart, true);
                    }

                    // Creating order
                    $result = $order->add();

                    if (!$result) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - Order cannot be created', 3, null, 'Cart', (int) $id_cart, true);
                        throw new PrestaShopException('Can\'t save Order');
                    }

                    // Amount paid by customer is not the right one -> Status = payment error
                    // We don't use the following condition to avoid the float precision issues : http://www.php.net/manual/en/language.types.float.php
                    // if ($order->total_paid != $order->total_paid_real)
                    // We use number_format in order to compare two string
                    if ($order_status->logable && number_format($cart_total_paid, _PS_PRICE_COMPUTE_PRECISION_) != number_format($amount_paid, _PS_PRICE_COMPUTE_PRECISION_)) {
                        $id_order_state = Configuration::get('PS_OS_ERROR');
                    }

                    $order_list[] = $order;

                    if (self::DEBUG_MODE) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - OrderDetail is about to be added', 1, null, 'Cart', (int) $id_cart, true);
                    }

                    // Insert new Order detail list using cart for the current order
                    $order_detail = new OrderDetail(null, null, $this->context);
                    $order_detail->createList($order, $this->context->cart, $id_order_state, $order->product_list, 0, true, $package_list[$id_address][$id_package]['id_warehouse']);
                    $order_detail_list[] = $order_detail;

                    if (self::DEBUG_MODE) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - OrderCarrier is about to be added', 1, null, 'Cart', (int) $id_cart, true);
                    }

                    // Adding an entry in order_carrier table
                    if (!is_null($carrier)) {
                        $order_carrier = new OrderCarrier();
                        $order_carrier->id_order = (int) $order->id;
                        $order_carrier->id_carrier = (int) $id_carrier;
                        $order_carrier->weight = (float) $order->getTotalWeight();
                        $order_carrier->shipping_cost_tax_excl = (float) $order->total_shipping_tax_excl;
                        $order_carrier->shipping_cost_tax_incl = (float) $order->total_shipping_tax_incl;
                        $order_carrier->add();
                    }
                }
            }

            // The country can only change if the address used for the calculation is the delivery address, and if multi-shipping is activated
            if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_delivery') {
                $this->context->country = $context_country;
            }

            if (!$this->context->country->active) {
                PrestaShopLogger::addLog('PaymentModule::validateOrder - Country is not active', 3, null, 'Cart', (int) $id_cart, true);
                throw new PrestaShopException('The order address country is not active.');
            }

            if (self::DEBUG_MODE) {
                PrestaShopLogger::addLog('PaymentModule::validateOrder - Payment is about to be added', 1, null, 'Cart', (int) $id_cart, true);
            }

            // Register Payment only if the order status validate the order
            if ($order_status->logable) {
                // $order is the last order loop in the foreach
                // The method addOrderPayment of the class Order make a create a paymentOrder
                // linked to the order reference and not to the order id
                if (isset($extra_vars['transaction_id'])) {
                    $transaction_id = $extra_vars['transaction_id'];
                } else {
                    $transaction_id = null;
                }

                if (!$order->addOrderPayment($amount_paid, null, $transaction_id)) {
                    PrestaShopLogger::addLog('PaymentModule::validateOrder - Cannot save Order Payment', 3, null, 'Cart', (int) $id_cart, true);
                    throw new PrestaShopException('Can\'t save Order Payment');
                }
            }

            // Next !
            $only_one_gift = false;
            $cart_rule_used = array();
            $products = $this->context->cart->getProducts();

            // Make sure CartRule caches are empty
            CartRule::cleanCache();
            foreach ($order_detail_list as $key => $order_detail) {
                /** @var OrderDetail $order_detail */
                $order = $order_list[$key];
                if (isset($order->id)) {
                    if (!$secure_key) {
                        $message .= '<br />' . $this->trans('Warning: the secure key is empty, check your payment account before validation', array(), 'Admin.Payment.Notification');
                    }
                    // Optional message to attach to this order
                    if (isset($message) & !empty($message)) {
                        $msg = new Message();
                        $message = strip_tags($message, '<br>');
                        if (Validate::isCleanHtml($message)) {
                            if (self::DEBUG_MODE) {
                                PrestaShopLogger::addLog('PaymentModule::validateOrder - Message is about to be added', 1, null, 'Cart', (int) $id_cart, true);
                            }
                            $msg->message = $message;
                            $msg->id_cart = (int) $id_cart;
                            $msg->id_customer = (int) ($order->id_customer);
                            $msg->id_order = (int) $order->id;
                            $msg->private = 1;
                            $msg->add();
                        }
                    }

                    // Insert new Order detail list using cart for the current order
                    //$orderDetail = new OrderDetail(null, null, $this->context);
                    //$orderDetail->createList($order, $this->context->cart, $id_order_state);
                    // Construct order detail table for the email
                    $products_list = '';
                    $virtual_product = true;

                    $product_var_tpl_list = array();
                    foreach ($order->product_list as $product) {
                        $price = Product::getPriceStatic((int) $product['id_product'], false, ($product['id_product_attribute'] ? (int) $product['id_product_attribute'] : null), 6, null, false, true, $product['cart_quantity'], false, (int) $order->id_customer, (int) $order->id_cart, (int) $order->{Configuration::get('PS_TAX_ADDRESS_TYPE')}, $specific_price, true, true, null, true, $product['id_customization']);
                        $price_wt = Product::getPriceStatic((int) $product['id_product'], true, ($product['id_product_attribute'] ? (int) $product['id_product_attribute'] : null), 2, null, false, true, $product['cart_quantity'], false, (int) $order->id_customer, (int) $order->id_cart, (int) $order->{Configuration::get('PS_TAX_ADDRESS_TYPE')}, $specific_price, true, true, null, true, $product['id_customization']);

                        $product_price = Product::getTaxCalculationMethod() == PS_TAX_EXC ? Tools::ps_round($price, 2) : $price_wt;

                        $product_var_tpl = array(
                            'id_product' => $product['id_product'],
                            'reference' => $product['reference'],
                            'name' => $product['name'] . (isset($product['attributes']) ? ' - ' . $product['attributes'] : ''),
                            'price' => Tools::displayPrice($product_price * $product['quantity'], $this->context->currency, false),
                            'quantity' => $product['quantity'],
                            'customization' => array()
                        );

                        if (isset($product['price']) && $product['price']) {
                            $product_var_tpl['unit_price'] = Tools::displayPrice($product_price, $this->context->currency, false);
                            $product_var_tpl['unit_price_full'] = Tools::displayPrice($product_price, $this->context->currency, false)
                                    . ' ' . $product['unity'];
                        } else {
                            $product_var_tpl['unit_price'] = $product_var_tpl['unit_price_full'] = '';
                        }

                        $customized_datas = Product::getAllCustomizedDatas((int) $order->id_cart, null, true, null, (int) $product['id_customization']);
                        if (isset($customized_datas[$product['id_product']][$product['id_product_attribute']])) {
                            $product_var_tpl['customization'] = array();
                            foreach ($customized_datas[$product['id_product']][$product['id_product_attribute']][$order->id_address_delivery] as $customization) {
                                $customization_text = '';
                                if (isset($customization['datas'][Product::CUSTOMIZE_TEXTFIELD])) {
                                    foreach ($customization['datas'][Product::CUSTOMIZE_TEXTFIELD] as $text) {
                                        $customization_text .= '<strong>' . $text['name'] . '</strong>: ' . $text['value'] . '<br />';
                                    }
                                }

                                if (isset($customization['datas'][Product::CUSTOMIZE_FILE])) {
                                    $customization_text .= $this->trans('%d image(s)', array(count($customization['datas'][Product::CUSTOMIZE_FILE])), 'Admin.Payment.Notification') . '<br />';
                                }

                                $customization_quantity = (int) $customization['quantity'];

                                $product_var_tpl['customization'][] = array(
                                    'customization_text' => $customization_text,
                                    'customization_quantity' => $customization_quantity,
                                    'quantity' => Tools::displayPrice($customization_quantity * $product_price, $this->context->currency, false)
                                );
                            }
                        }

                        $product_var_tpl_list[] = $product_var_tpl;
                        // Check if is not a virutal product for the displaying of shipping
                        if (!$product['is_virtual']) {
                            $virtual_product &= false;
                        }
                    } // end foreach ($products)

                    $product_list_txt = '';
                    $product_list_html = '';
                    if (count($product_var_tpl_list) > 0) {
                        $product_list_txt = $this->getEmailTemplateContent('order_conf_product_list.txt', Mail::TYPE_TEXT, $product_var_tpl_list);
                        $product_list_html = $this->getEmailTemplateContent('order_conf_product_list.tpl', Mail::TYPE_HTML, $product_var_tpl_list);
                    }

                    $cart_rules_list = array();
                    $total_reduction_value_ti = 0;
                    $total_reduction_value_tex = 0;
                    foreach ($cart_rules as $cart_rule) {
                        $package = array('id_carrier' => $order->id_carrier, 'id_address' => $order->id_address_delivery, 'products' => $order->product_list);
                        $values = array(
                            'tax_incl' => $cart_rule['obj']->getContextualValue(true, $this->context, CartRule::FILTER_ACTION_ALL_NOCAP, $package),
                            'tax_excl' => $cart_rule['obj']->getContextualValue(false, $this->context, CartRule::FILTER_ACTION_ALL_NOCAP, $package)
                        );

                        // If the reduction is not applicable to this order, then continue with the next one
                        if (!$values['tax_excl']) {
                            continue;
                        }

                        // IF
                        //  This is not multi-shipping
                        //  The value of the voucher is greater than the total of the order
                        //  Partial use is allowed
                        //  This is an "amount" reduction, not a reduction in % or a gift
                        // THEN
                        //  The voucher is cloned with a new value corresponding to the remainder
                        if (count($order_list) == 1 && $values['tax_incl'] > ($order->total_products_wt - $total_reduction_value_ti) && $cart_rule['obj']->partial_use == 1 && $cart_rule['obj']->reduction_amount > 0) {
                            // Create a new voucher from the original
                            $voucher = new CartRule((int) $cart_rule['obj']->id); // We need to instantiate the CartRule without lang parameter to allow saving it
                            unset($voucher->id);

                            // Set a new voucher code
                            $voucher->code = empty($voucher->code) ? substr(md5($order->id . '-' . $order->id_customer . '-' . $cart_rule['obj']->id), 0, 16) : $voucher->code . '-2';
                            if (preg_match('/\-([0-9]{1,2})\-([0-9]{1,2})$/', $voucher->code, $matches) && $matches[1] == $matches[2]) {
                                $voucher->code = preg_replace('/' . $matches[0] . '$/', '-' . (intval($matches[1]) + 1), $voucher->code);
                            }

                            // Set the new voucher value
                            if ($voucher->reduction_tax) {
                                $voucher->reduction_amount = ($total_reduction_value_ti + $values['tax_incl']) - $order->total_products_wt;

                                // Add total shipping amout only if reduction amount > total shipping
                                if ($voucher->free_shipping == 1 && $voucher->reduction_amount >= $order->total_shipping_tax_incl) {
                                    $voucher->reduction_amount -= $order->total_shipping_tax_incl;
                                }
                            } else {
                                $voucher->reduction_amount = ($total_reduction_value_tex + $values['tax_excl']) - $order->total_products;

                                // Add total shipping amout only if reduction amount > total shipping
                                if ($voucher->free_shipping == 1 && $voucher->reduction_amount >= $order->total_shipping_tax_excl) {
                                    $voucher->reduction_amount -= $order->total_shipping_tax_excl;
                                }
                            }
                            if ($voucher->reduction_amount <= 0) {
                                continue;
                            }

                            if ($this->context->customer->isGuest()) {
                                $voucher->id_customer = 0;
                            } else {
                                $voucher->id_customer = $order->id_customer;
                            }

                            $voucher->quantity = 1;
                            $voucher->reduction_currency = $order->id_currency;
                            $voucher->quantity_per_user = 1;
                            if ($voucher->add()) {
                                // If the voucher has conditions, they are now copied to the new voucher
                                CartRule::copyConditions($cart_rule['obj']->id, $voucher->id);
                                $orderLanguage = new Language((int) $order->id_lang);

                                $params = array(
                                    '{voucher_amount}' => Tools::displayPrice($voucher->reduction_amount, $this->context->currency, false),
                                    '{voucher_num}' => $voucher->code,
                                    '{firstname}' => $this->context->customer->firstname,
                                    '{lastname}' => $this->context->customer->lastname,
                                    '{id_order}' => $order->reference,
                                    '{order_name}' => $order->getUniqReference()
                                );
                                Mail::Send(
                                        (int) $order->id_lang, 'voucher', Context::getContext()->getTranslator()->trans(
                                                'New voucher for your order %s', array($order->reference), 'Emails.Subject', $orderLanguage->locale
                                        ), $params, $this->context->customer->email, $this->context->customer->firstname . ' ' . $this->context->customer->lastname, null, null, null, null, _PS_MAIL_DIR_, false, (int) $order->id_shop
                                );
                            }

                            $values['tax_incl'] = $order->total_products_wt - $total_reduction_value_ti;
                            $values['tax_excl'] = $order->total_products - $total_reduction_value_tex;
                            if (1 == $voucher->free_shipping) {
                                $values['tax_incl'] += $order->total_shipping_tax_incl;
                                $values['tax_excl'] += $order->total_shipping_tax_excl;
                            }
                        }
                        $total_reduction_value_ti += $values['tax_incl'];
                        $total_reduction_value_tex += $values['tax_excl'];

                        $order->addCartRule($cart_rule['obj']->id, $cart_rule['obj']->name, $values, 0, $cart_rule['obj']->free_shipping);

                        if ($id_order_state != Configuration::get('PS_OS_ERROR') && $id_order_state != Configuration::get('PS_OS_CANCELED') && !in_array($cart_rule['obj']->id, $cart_rule_used)) {
                            $cart_rule_used[] = $cart_rule['obj']->id;

                            // Create a new instance of Cart Rule without id_lang, in order to update its quantity
                            $cart_rule_to_update = new CartRule((int) $cart_rule['obj']->id);
                            $cart_rule_to_update->quantity = max(0, $cart_rule_to_update->quantity - 1);
                            $cart_rule_to_update->update();
                        }

                        $cart_rules_list[] = array(
                            'voucher_name' => $cart_rule['obj']->name,
                            'voucher_reduction' => ($values['tax_incl'] != 0.00 ? '-' : '') . Tools::displayPrice($values['tax_incl'], $this->context->currency, false)
                        );
                    }

                    $cart_rules_list_txt = '';
                    $cart_rules_list_html = '';
                    if (count($cart_rules_list) > 0) {
                        $cart_rules_list_txt = $this->getEmailTemplateContent('order_conf_cart_rules.txt', Mail::TYPE_TEXT, $cart_rules_list);
                        $cart_rules_list_html = $this->getEmailTemplateContent('order_conf_cart_rules.tpl', Mail::TYPE_HTML, $cart_rules_list);
                    }

                    // Specify order id for message
                    $old_message = Message::getMessageByCartId((int) $this->context->cart->id);
                    if ($old_message && !$old_message['private']) {
                        $update_message = new Message((int) $old_message['id_message']);
                        $update_message->id_order = (int) $order->id;
                        $update_message->update();

                        // Add this message in the customer thread
                        $customer_thread = new CustomerThread();
                        $customer_thread->id_contact = 0;
                        $customer_thread->id_customer = (int) $order->id_customer;
                        $customer_thread->id_shop = (int) $this->context->shop->id;
                        $customer_thread->id_order = (int) $order->id;
                        $customer_thread->id_lang = (int) $this->context->language->id;
                        $customer_thread->email = $this->context->customer->email;
                        $customer_thread->status = 'open';
                        $customer_thread->token = Tools::passwdGen(12);
                        $customer_thread->add();

                        $customer_message = new CustomerMessage();
                        $customer_message->id_customer_thread = $customer_thread->id;
                        $customer_message->id_employee = 0;
                        $customer_message->message = $update_message->message;
                        $customer_message->private = 1;

                        if (!$customer_message->add()) {
                            $this->errors[] = $this->trans('An error occurred while saving message', array(), 'Admin.Payment.Notification');
                        }
                    }

                    if (self::DEBUG_MODE) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - Hook validateOrder is about to be called', 1, null, 'Cart', (int) $id_cart, true);
                    }

                    // Hook validate order
                    Hook::exec('actionValidateOrder', array(
                        'cart' => $this->context->cart,
                        'order' => $order,
                        'customer' => $this->context->customer,
                        'currency' => $this->context->currency,
                        'orderStatus' => $order_status
                    ));

                    foreach ($this->context->cart->getProducts() as $product) {
                        if ($order_status->logable) {
                            ProductSale::addProductSale((int) $product['id_product'], (int) $product['cart_quantity']);
                        }
                    }

                    if (self::DEBUG_MODE) {
                        PrestaShopLogger::addLog('PaymentModule::validateOrder - Order Status is about to be added', 1, null, 'Cart', (int) $id_cart, true);
                    }

                    // Set the order status
                    $new_history = new OrderHistory();
                    $new_history->id_order = (int) $order->id;
                    $new_history->changeIdOrderState((int) $id_order_state, $order, true);
                    $new_history->addWithemail(true, $extra_vars);

                    // Switch to back order if needed
                    if (Configuration::get('PS_STOCK_MANAGEMENT') &&
                            ($order_detail->getStockState() ||
                            $order_detail->product_quantity_in_stock < 0)) {
                        $history = new OrderHistory();
                        $history->id_order = (int) $order->id;
                        $history->changeIdOrderState(Configuration::get($order->valid ? 'PS_OS_OUTOFSTOCK_PAID' : 'PS_OS_OUTOFSTOCK_UNPAID'), $order, true);
                        $history->addWithemail();
                    }

                    unset($order_detail);

                    // Order is reloaded because the status just changed
                    $order = new Order((int) $order->id);

                    // Send an e-mail to customer (one order = one email)
                    if ($id_order_state != Configuration::get('PS_PAYPHONE_PENDING') && $id_order_state != Configuration::get('PS_OS_ERROR') && $id_order_state != Configuration::get('PS_OS_CANCELED') && $this->context->customer->id) {
                        $invoice = new Address((int) $order->id_address_invoice);
                        $delivery = new Address((int) $order->id_address_delivery);
                        $delivery_state = $delivery->id_state ? new State((int) $delivery->id_state) : false;
                        $invoice_state = $invoice->id_state ? new State((int) $invoice->id_state) : false;

                        $data = array(
                            '{firstname}' => $this->context->customer->firstname,
                            '{lastname}' => $this->context->customer->lastname,
                            '{email}' => $this->context->customer->email,
                            '{delivery_block_txt}' => $this->_getFormatedAddress($delivery, "\n"),
                            '{invoice_block_txt}' => $this->_getFormatedAddress($invoice, "\n"),
                            '{delivery_block_html}' => $this->_getFormatedAddress($delivery, '<br />', array(
                                'firstname' => '<span style="font-weight:bold;">%s</span>',
                                'lastname' => '<span style="font-weight:bold;">%s</span>'
                            )),
                            '{invoice_block_html}' => $this->_getFormatedAddress($invoice, '<br />', array(
                                'firstname' => '<span style="font-weight:bold;">%s</span>',
                                'lastname' => '<span style="font-weight:bold;">%s</span>'
                            )),
                            '{delivery_company}' => $delivery->company,
                            '{delivery_firstname}' => $delivery->firstname,
                            '{delivery_lastname}' => $delivery->lastname,
                            '{delivery_address1}' => $delivery->address1,
                            '{delivery_address2}' => $delivery->address2,
                            '{delivery_city}' => $delivery->city,
                            '{delivery_postal_code}' => $delivery->postcode,
                            '{delivery_country}' => $delivery->country,
                            '{delivery_state}' => $delivery->id_state ? $delivery_state->name : '',
                            '{delivery_phone}' => ($delivery->phone) ? $delivery->phone : $delivery->phone_mobile,
                            '{delivery_other}' => $delivery->other,
                            '{invoice_company}' => $invoice->company,
                            '{invoice_vat_number}' => $invoice->vat_number,
                            '{invoice_firstname}' => $invoice->firstname,
                            '{invoice_lastname}' => $invoice->lastname,
                            '{invoice_address2}' => $invoice->address2,
                            '{invoice_address1}' => $invoice->address1,
                            '{invoice_city}' => $invoice->city,
                            '{invoice_postal_code}' => $invoice->postcode,
                            '{invoice_country}' => $invoice->country,
                            '{invoice_state}' => $invoice->id_state ? $invoice_state->name : '',
                            '{invoice_phone}' => ($invoice->phone) ? $invoice->phone : $invoice->phone_mobile,
                            '{invoice_other}' => $invoice->other,
                            '{order_name}' => $order->getUniqReference(),
                            '{date}' => Tools::displayDate(date('Y-m-d H:i:s'), null, 1),
                            '{carrier}' => ($virtual_product || !isset($carrier->name)) ? $this->trans('No carrier', array(), 'Admin.Payment.Notification') : $carrier->name,
                            '{payment}' => Tools::substr($order->payment, 0, 255),
                            '{products}' => $product_list_html,
                            '{products_txt}' => $product_list_txt,
                            '{discounts}' => $cart_rules_list_html,
                            '{discounts_txt}' => $cart_rules_list_txt,
                            '{total_paid}' => Tools::displayPrice($order->total_paid, $this->context->currency, false),
                            '{total_products}' => Tools::displayPrice(Product::getTaxCalculationMethod() == PS_TAX_EXC ? $order->total_products : $order->total_products_wt, $this->context->currency, false),
                            '{total_discounts}' => Tools::displayPrice($order->total_discounts, $this->context->currency, false),
                            '{total_shipping}' => Tools::displayPrice($order->total_shipping, $this->context->currency, false),
                            '{total_wrapping}' => Tools::displayPrice($order->total_wrapping, $this->context->currency, false),
                            '{total_tax_paid}' => Tools::displayPrice(($order->total_products_wt - $order->total_products) + ($order->total_shipping_tax_incl - $order->total_shipping_tax_excl), $this->context->currency, false));

                        if (is_array($extra_vars)) {
                            $data = array_merge($data, $extra_vars);
                        }

                        // Join PDF invoice
                        if ((int) Configuration::get('PS_INVOICE') && $order_status->invoice && $order->invoice_number) {
                            $order_invoice_list = $order->getInvoicesCollection();
                            Hook::exec('actionPDFInvoiceRender', array('order_invoice_list' => $order_invoice_list));
                            $pdf = new PDF($order_invoice_list, PDF::TEMPLATE_INVOICE, $this->context->smarty);
                            $file_attachement['content'] = $pdf->render(false);
                            $file_attachement['name'] = Configuration::get('PS_INVOICE_PREFIX', (int) $order->id_lang, null, $order->id_shop) . sprintf('%06d', $order->invoice_number) . '.pdf';
                            $file_attachement['mime'] = 'application/pdf';
                        } else {
                            $file_attachement = null;
                        }

                        if (self::DEBUG_MODE) {
                            PrestaShopLogger::addLog('PaymentModule::validateOrder - Mail is about to be sent', 1, null, 'Cart', (int) $id_cart, true);
                        }

                        $orderLanguage = new Language((int) $order->id_lang);

                        if (Validate::isEmail($this->context->customer->email)) {
                            Mail::Send(
                                    (int) $order->id_lang, 'order_conf', Context::getContext()->getTranslator()->trans(
                                            'Order confirmation', array(), 'Emails.Subject', $orderLanguage->locale
                                    ), $data, $this->context->customer->email, $this->context->customer->firstname . ' ' . $this->context->customer->lastname, null, null, $file_attachement, null, _PS_MAIL_DIR_, false, (int) $order->id_shop
                            );
                        }
                    }

                    // updates stock in shops
                    if (Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT')) {
                        $product_list = $order->getProducts();
                        foreach ($product_list as $product) {
                            // if the available quantities depends on the physical stock
                            if (StockAvailable::dependsOnStock($product['product_id'])) {
                                // synchronizes
                                StockAvailable::synchronize($product['product_id'], $order->id_shop);
                            }
                        }
                    }

                    $order->updateOrderDetailTax();

                    // sync all stock
                    (new StockManager())->updatePhysicalProductQuantity(
                            (int) $order->id_shop, (int) Configuration::get('PS_OS_ERROR'), (int) Configuration::get('PS_OS_CANCELED'), null, (int) $order->id
                    );
                } else {
                    $error = $this->trans('Order creation failed', array(), 'Admin.Payment.Notification');
                    PrestaShopLogger::addLog($error, 4, '0000002', 'Cart', intval($order->id_cart));
                    die($error);
                }
            } // End foreach $order_detail_list
            // Use the last order as currentOrder
            if (isset($order) && $order->id) {
                $this->currentOrder = (int) $order->id;
            }

            if (self::DEBUG_MODE) {
                PrestaShopLogger::addLog('PaymentModule::validateOrder - End of validateOrder', 1, null, 'Cart', (int) $id_cart, true);
            }

            return true;
        } else {
            $error = $this->trans('Cart cannot be loaded or an order has already been placed using this cart', array(), 'Admin.Payment.Notification');
            PrestaShopLogger::addLog($error, 4, '0000001', 'Cart', intval($this->context->cart->id));
            die($error);
        }
    }

    /**
     * Constructor de clase
     */
    public function __construct() {
        $this->name = 'payphone';
        $this->tab = 'payments_gateways';
        $this->version = '3.1.0';
        $this->author = 'Payphone Developer';
        $this->controllers = array('payment', 'validation');
        $this->is_eu_compatible = 1;
        $this->bootstrap = true;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $config_values = Configuration::getMultiple($this->config);

        if (!empty($config_values['PAYPHONE_TOKEN']))
            $this->token = $config_values['PAYPHONE_TOKEN'];
        if (!empty($config_values['PAYPHONE_STORE']))
            $this->store = $config_values['PAYPHONE_STORE'];

        parent::__construct();
        $this->displayName = $this->translate('PayPhone button');
        $this->description = $this->translate('Accept payments for your products via payphone button.');
        $this->confirmUninstall = $this->translate('Are you sure about removing?');
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);

        if (!isset($this->token))
            $this->warning = $this->translate('You must configure token parameter for this module');
        if (!count(Currency::checkPaymentCurrencies($this->id)))
            $this->warning = $this->translate('No currency has been set for this module.');
    }

    /**
     * Registra los hook requeridos para el pago
     * Crea una tabla en la base de datos para registar los datos del pago
     * @return boolean
     */
    public function install() {
        $tab = new Tab();
        $tab->active = 1;
        $tab->name = array();
        $tab->class_name = 'AdminPayphone';

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Controlador Admin para comprobar el pago con PayPhone';
        }

        $tab->id_parent = -1;
        $tab->module = $this->name;
        $tab->add();

        if (parent::install() &&
                $this->registerHook('displayHeader') &&
                $this->registerHook('paymentOptions') &&
                $this->registerHook('paymentReturn') &&
                $this->registerHook('displayOrderDetail') &&
                $this->registerHook('displayAdminOrder') &&
                $this->_installDb() &&
                $this->_createOrderStates())
            return true;
        return false;
    }

    /**
     * Desinstala el módulo actual, eliminando las variables de configuración y la tabla en BD
     * @return boolean
     */
    public function uninstall() {
        $id_tab = (int) Tab::getIdFromClassName('AdminPayphone');
        $tab = new Tab((int) $id_tab);
        $result = $tab->delete();
        if (!$result)
            return false;
        return true;
    }

    /**
     * Ejecuta la creación de estados de orden para el botón de pagos
     */
    private function _createOrderStates() {
        $this->_createPayphonePaymentStatus($this->payphone_pending, '#03a3e3', '', false, false, '', false);
        $this->_createPayphonePaymentStatus($this->payphone_green, '#4dbe50', 'payment', true, true, true, true);
        $this->_createPayphonePaymentStatus($this->payphone_red, '#fb4f38', 'payment_error', false, true, false, true);
        return true;
    }

    /**
     * Crea un estado de orden de compra
     * @param type $array Arreglo con el nombre de la orden en diversos idiomas
     * @param type $color Color que mostrará la orden
     * @param type $template Plantilla para el envío de correo
     * @param type $invoice Determina si el estado genera factura
     * @param type $send_email Determina si se envía correo
     * @param type $paid Determina si la orden ha sido pagada
     * @param type $logable Determina si se escribe en el log 
     */
    private function _createPayphonePaymentStatus($array, $color, $template, $invoice, $send_email, $paid, $logable) {
        foreach ($array as $key => $value) {
            $ow_status = Configuration::get($key);
            if ($ow_status === false) {
                $order_state = new OrderState();
            } else
                $order_state = new OrderState((int) $ow_status);

            $langs = Language::getLanguages();

            foreach ($langs as $lang) {
                if ($lang['iso_code'] == 'es' || $lang['iso_code'] == 'en') {
                    $name = $value[$lang['iso_code']];
                    $name_clean = utf8_encode(html_entity_decode($name));
                    $order_state->name[$lang['id_lang']] = $name_clean;
                }
            }

            $order_state->invoice = $invoice;
            $order_state->send_email = $send_email;

            if ($template != '')
                $order_state->template = $template;

            if ($paid != '')
                $order_state->paid = $paid;

            $order_state->logable = $logable;
            $order_state->color = $color;
            $order_state->save();

            Configuration::updateValue($key, (int) $order_state->id);

            Tools::copy(dirname(__FILE__) . '/views/images/' . $key . '.gif', _PS_ROOT_DIR_ . '/img/os/' . (int) $order_state->id . '.gif');
        }
    }

    /**
     * Crea una tabla en la BD para registrar los datos del pago
     * @return type
     */
    private function _installDb() {
        return Db::getInstance()->Execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'payphone_transaction` (
                `id` INT(11) unsigned NOT NULL auto_increment,
                `transaction_id` INT(11) unsigned NOT NULL,
                `message` VARCHAR(255) NOT NULL,
                `authorization_code` VARCHAR(100) NULL,
                `phone_number` VARCHAR(20) NOT NULL,
                `transaction_status` VARCHAR(50) NOT NULL,
                `client_transaction_id` VARCHAR(50) NOT NULL,
                `client_user_id` VARCHAR(50) NULL,
                `deferred` TINYINT(1) NULL,
                `deferred_message` VARCHAR(255) NULL,
                `amount` VARCHAR(10) NOT NULL,
                `bin` VARCHAR(6) NULL,
                `card_bran` VARCHAR(30) NULL,
                `date_add` DATETIME NOT NULL,
                PRIMARY KEY (`id`)) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 AUTO_INCREMENT=1');
    }

    /**
     * Valida los variables de configuración
     */
    protected function _postValidation() {
        if (Tools::isSubmit('btnSubmit')) {
            foreach ($this->config_vars as $key => $value) {
                if (!Tools::getValue($key))
                    $this->_postErrors[] = $this->translate($value);
            }
        }
    }

    /**
     * Guarda los valores de la variables de configuración
     */
    protected function _postProcess() {
        if (Tools::isSubmit('btnSubmit')) {
            foreach ($this->config as $key) {
                Configuration::updateValue($key, Tools::getValue($key));
            }
        }
        $this->_html .= $this->displayConfirmation($this->translate('Settings updated'));
    }

    public function translate($text) {
        return $this->l($text, 'payphone');
    }

    /**
     * Devuelve el valor de las variables de configuración
     * @return array
     */
    public function getConfigFieldsValues() {
        $fields = array();
        foreach ($this->config as $key) {
            $fields[$key] = Tools::getValue($key, Configuration::get($key));
        }
        return $fields;
    }

    /**
     * Ejecuta el post del formulario de validación y muestra el mismo
     * @return html
     */
    public function getContent() {
        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors as $err)
                    $this->_html .= $this->displayError($err);
        } else
            $this->_html .= '<br />';
        $this->_html .= '<p><b>' . $this->trans('Prestashop response URL', array(), 'Modules.PayPhone.Admin') . ":</b> " . $this->context->link->getModuleLink('payphone', 'validation', array(), Configuration::get('PS_SSL_ENABLED')) . "</p>";
        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    public function hookdisplayHeader($params) {
        $this->context->controller->addCss(_PS_MODULE_DIR_ . 'payphone/views/css/payphone.css', 'all');
        $this->context->controller->addCss(_PS_MODULE_DIR_ . 'payphone/views/css/error.css', 'all');
    }

    /**
     * Hook que se ejecuta al efectuar el pago con el botón de pagos PayPhone para la versión 1.7 o superior
     * @param type $params
     * @return type
     */
    public function hookPaymentOptions($params) {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        unset($this->context->cookie->payphone_cart_id);

        $this->context->smarty->assign(array(
            'module_dir' => _MODULE_DIR_
        ));

        $paymentOptions = array();
        $payPhoneButtonOption = new PaymentOption();
        $payPhoneButtonOption->setCallToActionText($this->trans('Pagar con PayPhone', array(), 'Modules.PayPhone.Admin'))
                ->setAction($this->context->link->getModuleLink('payphone', 'payment', array('payment_type' => PaymentType::PAYPHONE), Configuration::get('PS_SSL_ENABLED')))
                ->setAdditionalInformation($this->fetch('module:payphone/views/templates/hook/payment17.tpl'));
        $paymentOptions[] = $payPhoneButtonOption;
        $cardOption = new PaymentOption();
        $cardOption->setCallToActionText($this->trans('Pagar con tarjeta', array('paym'), 'Modules.PayPhone.Admin'))
                ->setAction($this->context->link->getModuleLink('payphone', 'payment', array('payment_type' => PaymentType::CARD), Configuration::get('PS_SSL_ENABLED')))
                ->setAdditionalInformation($this->fetch('module:payphone/views/templates/hook/payment_card.tpl'));
        $paymentOptions[] = $cardOption;
        return $paymentOptions;
    }

    /**
     * Hook que se ejecuta para mostrar los resultados del pago con el botón de pagos PayPhone
     * @param type $params
     * @return boolean
     */
    public function hookPaymentReturn($params) {
        if (!$this->active) {
            return;
        }
        $paramOrder = $params['order'];
        if (!isset($paramOrder) || ($paramOrder->module != $this->name))
            return false;

        if (isset($paramOrder) && Validate::isLoadedObject($paramOrder) && isset($paramOrder->valid) && isset($paramOrder->reference)) {
            $sql = new DbQuery();
            $sql->select('*');
            $sql->from('payphone_transaction', 'pt');
            $sql->where('pt.client_user_id = ' . $paramOrder->id_cart);
            $data = Db::getInstance()->executeS($sql);

            $format = 'Y-m-d H:i:s';
            $date = DateTime::createFromFormat($format, $data[0]['date_add']);
            $total = Tools::displayPrice(floatval($data[0]['amount'] / 100), null, false);
            $this->smarty->assign(array(
                'transaction_id' => $data[0]['transaction_id'],
                'status' => $data[0]['transaction_status'],
                'date' => $date->format('d/m/Y H:i:s'),
                'amount' => $total,
                'authorization_code' => $data[0]['authorization_code'],
                'message' => $data[0]['message'],
                'phone_number' => $data[0]['phone_number'],
                'module_dir' => _MODULE_DIR_,
            ));
            $result = $this->fetch('module:payphone/views/templates/hook/order-confirmation17.tpl');
            return $result;
        }
    }

    /**
     * Hook que se ejecuta al mostrar los detalles de la orden desde el Admin, cuyo pago se realizó con PayPhone
     * @param type $params
     * @return html
     */
    public function hookDisplayAdminOrder($params) {
        $order = new Order($params['id_order']);
        if (!in_array($order->current_state, array(Configuration::get('PS_PAYPHONE_APPROVED'), Configuration::get('PS_PAYPHONE_PENDING'), Configuration::get('PS_PAYPHONE_REJECTED'))))
            return;
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('payphone_transaction', 'pt');
        $sql->where('pt.client_transaction_id = "' . $order->reference . '"');
        $data = Db::getInstance()->executeS($sql);
        $status = ($data == false) ? 'Pending' : $data[0]['transaction_status'];

        $link = $this->context->link->getAdminLink('AdminPayphone', true) . '&id_order=' . $order->id;
        $this->smarty->assign(array(
            'transaction_id' => $data[0]['transaction_id'],
            'authorization_code' => $data[0]['authorization_code'],
            'phone_number' => $data[0]['phone_number'],
            'message' => $data[0]['message'],
            'bin' => $data[0]['bin'],
            'status' => $status,
            'card_brand' => $data[0]['card_bran'],
            'check_payment_url' => $link,
        ));
        return $this->display(__FILE__, 'views/templates/hook/admin-order17.tpl');
    }

    /**
     * Hook que se ejecuta al mostrar los detalles de la orden al cliente cuyo pago se realizó con PayPhone
     * @param type $order
     * @return html
     */
    public function hookDisplayOrderDetail($order) {
        if (!in_array($order['order']->current_state, array(Configuration::get('PS_PAYPHONE_APPROVED'), Configuration::get('PS_PAYPHONE_PENDING'), Configuration::get('PS_PAYPHONE_REJECTED'))))
            return;
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('payphone_transaction', 'pt');
        $sql->where('pt.client_transaction_id = "' . $order['order']->reference . '"');
        $data = Db::getInstance()->executeS($sql);
        if ($data == false) {
            $status = 'Pending';
            $data[0]['transaction_id'] = "---";
            $data[0]['authorization_code'] = "---";
            $data[0]['phone_number'] = "---";
            $data[0]['message'] = "---";
            $data[0]['bin'] = "---";
            $data[0]['card_bran'] = "---";
        } else
            $status = $data[0]['transaction_status'];

        $link = $this->context->link->getModuleLink('payphone', 'validation', array('id_order' => $order['order']->id), Configuration::get('PS_SSL_ENABLED'));
        $this->smarty->assign(array(
            'transaction_id' => $data[0]['transaction_id'],
            'authorization_code' => $data[0]['authorization_code'],
            'phone_number' => $data[0]['phone_number'],
            'message' => $data[0]['message'],
            'bin' => $data[0]['bin'],
            'status' => $status,
            'card_brand' => $data[0]['card_bran'],
            'check_payment_url' => $link,
        ));
        return $this->display(__FILE__, 'views/templates/hook/order-detail17.tpl');
    }

    /**
     * Verifica si el plugin actual tiene habilitado la misma moneda que el carro de compra
     * @param type $cart Datos del carrito de compra
     * @return boolean
     */
    public function checkCurrency($cart) {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module))
            foreach ($currencies_module as $currency_module)
                if ($currency_order->id == $currency_module['id_currency'])
                    return true;
        return false;
    }

    /**
     * Establece los campos del formulario de configuración usando el API de formularios
     * @return type
     */
    public function renderForm() {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->translate('Variables'),
                    'icon' => 'icon-envelope'
                ),
                'input' => array(
                    array(
                        'type' => 'textarea',
                        'label' => $this->translate('Token'),
                        'name' => 'PAYPHONE_TOKEN',
                        'required' => true
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->translate('Store id'),
                        'name' => 'PAYPHONE_STORE',
                        'desc' => $this->translate('Store id in payphone')
                    ),
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Actions'),
                    'class' => 'btn btn-default pull-right'
                )
            ),
        );
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
        return $helper->generateForm(array($fields_form));
    }

    /**
     * Realiza un post al API de pago de PayPhone para verificar el estado de una transacción.
     * Actualiza la orden
     * @param type $id_order
     * @return boolean Si la llamada fue satisfactoria o no
     */
    public function updatePendingTransaction($id_order) {
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order))
            return false;
        $config = ConfigurationManager::Instance();
        $config->Token = $this->token;
        $pb = new PayphoneButton();
        try {
            $data = $pb->GetSaleByClientId($order->reference);
            if (is_array($data)) {
                $data = $data[0];
                $status = Configuration::get('PS_PAYPHONE_PENDING');
                if ($data->statusCode == 3)
                    $status = Configuration::get('PS_PAYPHONE_APPROVED');
                if ($data->statusCode == 2)
                    $status = Configuration::get('PS_PAYPHONE_REJECTED');
                if ($status != Configuration::get('PS_PAYPHONE_PENDING')) {
                    ShopUrl::cacheMainDomainForShop((int) $order->id_shop);
                    $order_state = new OrderState($status);
                    if (!Validate::isLoadedObject($order_state)) {
                        $this->errors[] = Tools::displayError('The new order status is invalid.');
                    }
                    $current_order_state = $order->getCurrentOrderState();
                    $history = new OrderHistory();
                    $history->id_order = $order->id;
                    $use_existings_payment = false;
                    if (!$order->hasInvoice()) {
                        $use_existings_payment = true;
                    }
                    $history->changeIdOrderState((int) $order_state->id, $order, $use_existings_payment);
                    $history->addWithemail(true);
                    $write = array();
                    $write['transaction_id'] = (int) $data->transactionId;
                    $write['message'] = ($data->statusCode == 3) ? pSQL("Pago recibido por el vendedor") : pSQL("Pago no efectuado");
                    $write['phone_number'] = pSQL($data->phoneNumber);
                    $write['transaction_status'] = pSQL($data->transactionStatus);
                    $write['client_transaction_id'] = pSQL($data->clientTransactionId);
                    $write['client_user_id'] = pSQL($order->id_cart);
                    $write['deferred'] = $data->deferred;
                    $write['amount'] = $data->amount;
                    $write['date_add'] = date("Y-m-d H:i:s");
                    if ($data->deferred)
                        $write['deferred_message'] = pSQL($data->deferredMessage);
                    if ($data->statusCode == 3) {
                        $write['bin'] = pSQL($data->bin);
                        $write['card_bran'] = pSQL($data->cardBrand);
                        $write['authorization_code'] = pSQL($data->authorizationCode);
                    }
                    Db::getInstance()->insert('payphone_transaction', $write);
                }
                return true;
            } else {
                if ($data->error->message == 'Transacción no encontrada') {
                    ShopUrl::cacheMainDomainForShop((int) $order->id_shop);
                    $order_state = new OrderState(Configuration::get('PS_OS_CANCELED'));

                    if (!Validate::isLoadedObject($order_state)) {
                        $this->errors[] = Tools::displayError('The new order status is invalid.');
                    }
                    $current_order_state = $order->getCurrentOrderState();
                    $history = new OrderHistory();
                    $history->id_order = $order->id;
                    $use_existings_payment = false;
                    if (!$order->hasInvoice()) {
                        $use_existings_payment = true;
                    }
                    $history->changeIdOrderState((int) $order_state->id, $order, $use_existings_payment);
                    $history->addWithemail(true);
                    return true;
                } else {
                    return $data->error->errors[0];
                }
            }
        } catch (PayPhoneWebException $e) {
            $this->context->cookie->payphone_cart_id = (int) $order->id_cart;
            return $e->ErrorList[0]->message;
        } catch (Exception $e) {
            $this->context->cookie->payphone_cart_id = (int) $order->id_cart;
            return $e->getMessage();
        }
    }

}
